/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dataPool.api.dataRule.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dataPool.db.entity.DataResourcePackage;
import net.northking.cctp.dataPool.dto.dataRule.DataResourcePackageDetailDto;
import net.northking.cctp.dataPool.dto.dataRule.DataResourcePackageQueryDto;
import net.northking.cctp.dataPool.dto.dataRule.DataResourcePackageUpdateDto;
import org.springframework.web.multipart.MultipartFile;

/**
 * 资源包Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface DataResourcePackageApiService extends ExcelService
{
  /**
   * 查询资源包列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<DataResourcePackageDetailDto> pagingQuery(QueryByPage<DataResourcePackageQueryDto> queryByPage);

  /**
   * 更新资源包
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(DataResourcePackageUpdateDto dto) throws IllegalAccessException;

  /**
   * 删除资源包
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

  /**
   * 上传资源包
   * @return
   */
  DataResourcePackage uploadResource(MultipartFile file,String proId,String memo);
}
