/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2022 . All rights reserved.
*
*/
package net.northking.cctp.dataPool.dto.dataRule;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.io.Serializable;

/**
* Restful接口--新增资源--数据传输对象：数据规则表
*
* createdate:  2022-10-24 17:01:18  <br>
* @author:  maven-cctp-plugin  <br>
* @since:  1.0 <br>
*/
@JsonSerialize
@ApiModel(description = "数据规则表")
public class DataRuleAddDto implements Serializable
{
    /**
     * 规则名称<br>
     */
    @ApiModelProperty("规则名称")
    @Size(max = 32, message="规则名称-name: 数据长度不能 > 32" )
    @NotBlank(message = "规则名称不能为空")
    private String name;
    /**
     * 规则类型 0-脚本语言 1-正则表达式<br>
     */
    @ApiModelProperty("规则类型 0-脚本语言 1-正则表达式")
    @Size(max = 1, message="规则类型-type: 数据长度不能 > 1" )
    @NotBlank(message = "规则类型不能为空")
    private String type;
    /**
     * 正则表达式、脚本编码<br>
     */
    @ApiModelProperty("正则表达式、脚本编码")
    @Size(max = 65535, message="正则表达式、脚本编码-contentExpression: 数据长度不能 > 65535" )
    private String contentExpression;
    /**
     * 系统id<br>
     */
    @ApiModelProperty("系统id")
    @Size(max = 32, message="系统id-proId: 数据长度不能 > 32" )
    private String proId;
    /**
     * 资源包集合<br>
     */
    @ApiModelProperty("资源包集合")
    @Size(max = 1024, message="资源包集合-jarIds: 数据长度不能 > 1024" )
    private String jarIds;

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getContentExpression() {
        return contentExpression;
    }

    public void setContentExpression(String contentExpression) {
        this.contentExpression = contentExpression;
    }

    public String getProId() {
        return proId;
    }

    public void setProId(String proId) {
        this.proId = proId;
    }

    public String getJarIds() {
        return jarIds;
    }

    public void setJarIds(String jarIds) {
        this.jarIds = jarIds;
    }
}