/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2023 . All rights reserved.
*
*/
package net.northking.cctp.project.api.env.service;

import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.project.dto.env.*;

import java.util.List;
import java.util.Map;

/**
 * 系统环境数据源信息Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface CpProjectEnvDsApiService extends ExcelService
{
  /**
   * 查询系统环境数据源信息列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<CpProjectEnvDsDetailDto> pagingQuery(QueryByPage<CpProjectEnvDsQueryDto> queryByPage);

  /**
   * 新增系统环境数据源信息
   *
   * @param dto         接口数据传输对象
   * @return 数据库主键
   */
  String add(CpProjectEnvDsAddDto dto);

  /**
   * 更新系统环境数据源信息
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(CpProjectEnvDsUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取系统环境数据源信息
   * @param uuid 数据库主键
   * @return 接口数据传输对象
   */
  CpProjectEnvDsDetailDto findByPK(String uuid);

  /**
   * 删除系统环境数据源信息
   * @param uuid  数据库主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);


    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<CpProjectEnvDsUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  /**
   * 系统环境数据源信息下拉选项
   * @param projectId 系统id
   * @param envId 环境id
   * @return 选项集
   */
  List<CpProjectEnvDsSelectDto> findSelect(String projectId, String envId);

  /**
   * 测试连接
   * @param connDto 连接参数
   * @return 结果
   */
  Boolean checkConnection(CpProjectEnvDsConnDto connDto);

  /**
   * 根据id测试连接
   * @param uuid 数据源id
   * @return 结果
   */
  Boolean checkConnectionById(String uuid);

  /**
   * sql执行
   * @param executeDto 执行参数
   * @return 结果集
   */
  Map<String, Object> executeSql(CpProjectEnvDsExecuteDto executeDto);

  /**
   * 执行组件sql
   * @param execDto 执行参数
   * @return 连接信息
   */
  String executeModuleSql(CpProjectEnvDsModuleExecDto execDto);

  /**
   * 同步系统环境数据源信息
   * @param projectId 系统id
   * @param envIds 环境id集合
   */
    void syncProjectEnvDs(String projectId, List<String> envIds);

}
