/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.testbug.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.testbug.dto.OverallViewIndexAddDto;
import net.northking.cctp.testbug.dto.OverallViewIndexDetailDto;
import net.northking.cctp.testbug.dto.OverallViewIndexQueryDto;
import net.northking.cctp.testbug.dto.OverallViewIndexUpdateDto;

import java.util.List;

/**
 * 跨项目级视图索引信息表Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface OverallViewIndexApiService extends ExcelService
{
  /**
   * 查询跨项目级视图索引信息表列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<OverallViewIndexDetailDto> pagingQuery(QueryByPage<OverallViewIndexQueryDto> queryByPage);

  /**
   * 新增跨项目级视图索引信息表
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(OverallViewIndexAddDto dto);

  /**
   * 更新跨项目级视图索引信息表
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(OverallViewIndexUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取跨项目级视图索引信息表
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  OverallViewIndexDetailDto findByPK(String uuid);

  /**
   * 删除跨项目级视图索引信息表
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<OverallViewIndexAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<OverallViewIndexUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

    OverallViewIndexDetailDto oneQuery(String userId,String recordType);
}
