/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.common.http.ResultWrapper;

import net.northking.cctp.common.security.authentication.NKSecurityContext;
import net.northking.cctp.dw.api.service.MtTestPointStatisticsApiService;
import net.northking.cctp.dw.dto.ChartDataGeneQueryDto;
import net.northking.cctp.dw.dto.MtTestPointStatisticsDetailDto;
import net.northking.cctp.dw.dto.MtTestPointStatisticsQueryDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(tags = "需求分析统计图")
@RequestMapping("/v1")
@RestController
public class MtTestPointStatisticsQueryCtrl
{
  /**
   * 日志
   */
  private final static Logger logger = LoggerFactory.getLogger(MtTestPointStatisticsQueryCtrl.class);

  @Autowired
  private MtTestPointStatisticsApiService apiService;

  /**
   * 查询用户列表
   *
   * @param queryByPage  查询条件
   * @return 数据列表
   */
  @ApiOperation(value = "分页查询列表：需求分析统计图")
  @PostMapping(value = "/paging/query/MtTestPointStatisticss",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<Pagination<MtTestPointStatisticsDetailDto>> pagingQuery(
      @RequestBody @Validated QueryByPage<MtTestPointStatisticsQueryDto> queryByPage)
  {
    ResultWrapper<Pagination<MtTestPointStatisticsDetailDto>> wrapper = new ResultWrapper<>();
    Pagination<MtTestPointStatisticsDetailDto> page = apiService.pagingQuery(queryByPage);

    return wrapper.success(page);
  }

  @ApiOperation(value = "导出Excel：需求分析统计图")
  @PostMapping(value = "/excel/export/MtTestPointStatisticss",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void exportExcel(
      @RequestBody @Validated QueryByPage<MtTestPointStatisticsQueryDto> queryByPage,
      HttpServletResponse response)
  {
    // 定义导出的Excel在客户端浏览器上提示下载的文件名，不写后缀
    String fileName = "CCTP-需求分析统计图";
    apiService.exportExcel(response, queryByPage, fileName);
  }

  @ApiOperation(value = "下载Excel模板：需求分析统计图")
  @PostMapping(value = "/excel/template/MtTestPointStatisticss",
      produces = MediaType.APPLICATION_OCTET_STREAM_VALUE,
      consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public void downloadTemplate(HttpServletResponse response)
  {
    String fileName = "CCTP模板-需求分析统计图";
    apiService.downloadTemplate(response, fileName);
  }

  @ApiOperation(value = "导入Excel：需求分析统计图")
  @PostMapping(value = "/excel/import/MtTestPointStatisticss",
      produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
      consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
  public ResultWrapper<Integer> importExcel(
      @RequestParam("file") MultipartFile file)
  {
    ResultWrapper<Integer> wrapper = new ResultWrapper<>();
    int count = apiService.importExcel(file);

    return wrapper.success(count);
  }

  /**
   * 查询列表
   *
   * @param
   * @return 数据列表
   */
  @ApiOperation(value = "查询列表：需求分析统计图")
  @PostMapping(value = "/list/query/MtTestPointStatisticss",
          produces = MediaType.APPLICATION_JSON_UTF8_VALUE,
          consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
  public ResultWrapper<List<MtTestPointStatisticsDetailDto>> listQuery(@RequestBody ChartDataGeneQueryDto chartDataGeneQueryDto)
  {
    ResultWrapper<List<MtTestPointStatisticsDetailDto>> wrapper = new ResultWrapper<>();
    chartDataGeneQueryDto.setTenantId(NKSecurityContext.getTenantId());
    List<MtTestPointStatisticsDetailDto> page = apiService.listQuery(chartDataGeneQueryDto);

    return wrapper.success(page);
  }

}


