/*
* Copyright (c) 京北方信息技术股份有限公司 Corporation 2021 . All rights reserved.
*
*/
package net.northking.cctp.dw.api.service;

import net.northking.cctp.common.db.excel.ExcelService;
import net.northking.cctp.common.db.Pagination;
import net.northking.cctp.common.http.QueryByPage;
import net.northking.cctp.dw.dto.BugLifeCycleChartAddDto;
import net.northking.cctp.dw.dto.BugLifeCycleChartDetailDto;
import net.northking.cctp.dw.dto.BugLifeCycleChartQueryDto;
import net.northking.cctp.dw.dto.BugLifeCycleChartUpdateDto;

import java.util.List;

/**
 * 缺陷生命周期Restful逻辑接口
 * @author maven-cctp-plugin
 * @since 1.0
 */
public interface BugLifeCycleChartApiService extends ExcelService
{
  /**
   * 查询缺陷生命周期列表
   *
   * @param queryByPage  查询条件
   * @return 查询结果列表
   */
  Pagination<BugLifeCycleChartDetailDto> pagingQuery(QueryByPage<BugLifeCycleChartQueryDto> queryByPage);

  /**
   * 新增缺陷生命周期
   *
   * @param dto         接口数据传输对象
   * @return 主键
   */
  String add(BugLifeCycleChartAddDto dto);

  /**
   * 更新缺陷生命周期
   *
   * @param dto         接口数据传输对象
   * @return 修改的记录数量
   */
  Integer updateByPK(BugLifeCycleChartUpdateDto dto) throws IllegalAccessException;

  /**
   * 获取缺陷生命周期
   * @param uuid 主键
   * @return 接口数据传输对象
   */
  BugLifeCycleChartDetailDto findByPK(String uuid);

  /**
   * 删除缺陷生命周期
   * @param uuid  主键
   * @return 删除的记录数量
   */
  Integer deleteByPK(String uuid);

    /**
     * 批量新增
     *
     * @param batch 集合
     * @return 成功的数量
     */
    List<String> addBatch(List<BugLifeCycleChartAddDto> batch);

    /**
     * 批量更新
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer updateBatch(List<BugLifeCycleChartUpdateDto> batch) throws IllegalAccessException;

    /**
     * 批量删除
     *
     * @param batch 集合
     * @return 成功的数量
     */
    Integer deleteBatch(List<Object> batch);

  List<BugLifeCycleChartDetailDto> listQuery(String proId,String tenantId);
}
